import java.util.Iterator;
import java.util.Vector;

/*! Copyright (C) [2010] [David Mainzer]
 * 
 *  This program is free software; you can redistribute it and/or modify it under the terms 
 *  of the GNU General Public License as published by the Free Software Foundation; either 
 *  version 3 of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU General Public License for more details.
 *  
 *	You should have received a copy of the GNU General Public License along with this program; 
 *	if not, see <http://www.gnu.org/licenses/>.  
 */


/**  Wenn etwas in dieser Klasse fehlt, fgt dies ruhig hinzu :)
 *
 *  N      N      N
 *    \    |    /
 *      2  |  2
 *        \|/
 *  N -1-  P  -1- N 
 *        /|\
 *      2  |  2
 *    /    |    \
 *  N      N      N
 *  
 *  WITH:
 *  P = MassPoint
 *  N = Neighbors
 *  1 = structural springs
 *  2 = shear springs
 */

public class MassPoint {
	// MassPoint ID (given by the wrl-file from coordIndex)
	private int ID;
	
	// store the OLD position
	private Vector3 oldPos = new Vector3();
	// store the CURRENT position
	private Vector3 curPos = new Vector3();
	// store the NEW position (adding force to curPos)
	private Vector3 newPos = new Vector3();
	
	// structural springs
	private Vector<Integer> m_structural_springs = new Vector<Integer>();
	// shear springs
	private Vector<Integer> m_shear_springs = new Vector<Integer>();
	
	// length of the springs
	final public float m_structural_spring_length = 0.11F;
	final public float m_shear_spring_length = (float) Math.sqrt( 0.0242 );
	
	// mass of MassPoint
	final public float mass = 0.0005F;

	// force affect to MassPoint
	private Vector3 force = new Vector3();
	
	// constructors
	MassPoint( int ID )
	{
		this.ID     = ID;
	}
	
	MassPoint( int ID, float x, float y, float z )
	{
		this.ID     = ID;

		this.oldPos.x(x);
		this.oldPos.y(y);
		this.oldPos.z(z);
		
		this.newPos.x(x);
		this.newPos.y(y);
		this.newPos.z(z);
		
		this.curPos.x(x);
		this.curPos.y(y);
		this.curPos.z(z);
	}
	
	MassPoint( int ID, float [] pos )
	{
		if (pos.length != 3)
		{
			System.err.println("pos vector has " + pos.length + "elements but only 3 accepted.");
			System.exit(-1);
		}
		this.ID     = ID;
		this.oldPos.x(pos[0]);
		this.oldPos.y(pos[1]);
		this.oldPos.z(pos[2]);
		
		this.newPos.x(pos[0]);
		this.newPos.y(pos[1]);
		this.newPos.z(pos[2]);
		
		this.curPos.x(pos[0]);
		this.curPos.y(pos[1]);
		this.curPos.z(pos[2]);
	}
	
	/** add structural connection to mass-point
	 * 
	 * @param ID
	 */
	public void addStructuralSpringConnection (int ID)
	{
		if ( ! this.m_structural_springs.contains(ID) )
			this.m_structural_springs.add(ID);
	}
	
	/** return the structural connections
	 * 
	 * @return structural springs
	 */
	public Vector<Integer> getStructuralSpringConnection()
	{
		return this.m_structural_springs;
	}
	
	/** return the shear connections
	 * 
	 * @return
	 */
	public Vector<Integer> getShearSpringConnection()
	{
		return this.m_shear_springs;
	}
	
	/** add shear connection to mass-point
	 * 
	 * @param ID
	 */
	public void addShearSpringConnection (int ID)
	{
		if ( ! this.m_shear_springs.contains(ID) )
			this.m_shear_springs.add(ID);
	}
	
	public float stiffness (float length)
	{
		// hier muss die Federkonstante berechnet werden
		return length;
	}
	
	public float damper (float length)
	{
		// hier muss die Dmpfung berechnet werden
		return length;
	}
	
	/** add force to the force vector of the mass-point
	 * 
	 * @param force
	 */
	public void addForce( Vector3 force )
	{
		this.force = this.force.add(force);
	}
	
	/** update current position by adding the force and store as new position, create a new empty force vector
	 *  and set new as current position
	 */
	public void updateCurPos ()
	{
		this.newPos = this.curPos.add(this.force);
		this.force = new Vector3();
		this.curPos = this.newPos;
	}
	
	/** print all spring connection for this mass-point
	 * 
	 */
	public void printSpringConnections ()
	{
		System.out.println("connections for spring-ID: " + ID );
		System.out.print( "structural springs: " );
		Iterator<Integer> iter = m_structural_springs.iterator();
		while (iter.hasNext()) {
    		System.out.print(" " + iter.next().intValue() );
    	}
		System.out.print("\n");
		System.out.print( "shear springs: " );
		Iterator<Integer> iter_shear = m_shear_springs.iterator();
		while (iter_shear.hasNext()) {
    		System.out.print(" " + iter_shear.next().intValue() );
    	}
		System.out.print("\n");
		System.out.print("\n");
	}
	
///////////////////////
// GETTER and SETTER //
///////////////////////
	
	/** return the id of the MassPoint
	 *
	 * @return ID -- MassPoint ID
	 */
	public int getID()
	{
		return this.ID;
	}

	/** return the current position of the MassPoint
	 *
	 * @return curPos -- the current position of MassPoint
	 */
	public Vector3 getCurPos()
	{
		return this.curPos;
	}
	
	/** return the x coordinate of the current position of MassPoint
	 *
	 * @return curPos.x() -- the x coordinate of the current position
	 */
	public float getX()
	{
		return this.curPos.x();
	}
	
	/** return the y coordinate of the current position of MassPoint
	 *
	 * @return curPos.y() -- the y coordinate of the current position
	 */
	public float getY()
	{
		return this.curPos.y();
	}
	
	/** return the z coordinate of the current position of MassPoint
	 *
	 * @return curPos.z() -- the z coordinate of the current position
	 */
	public float getZ()
	{
		return this.curPos.z();
	}
	
	/** owerwrite the current position (have you backup the current position as old position)
	 *
	 * @warning backup the current position as old position before
	 * @param x -- new x coordinate
	 * @param y -- new y coordinate
	 * @param z -- new z coordinate
	 */
	public void setCurPos(float x, float y, float z)
	{
		this.curPos.x(x);
		this.curPos.y(y);
		this.curPos.z(z);
	}
	
	/** backup the current position as old position
	 *
	 */
	public void setOldPos( )
	{
		this.oldPos = this.curPos;
	}
	
}
